package hu.bme.mit.szolgint.osgi.demo;

import hu.bme.mit.szolgint.osgi.demo.entities.Book;
import hu.bme.mit.szolgint.osgi.demo.entities.EntityManagerProvider;
import hu.bme.mit.szolgint.osgi.demo.entities.User;
import hu.bme.mit.szolgint.osgi.demo.exceptions.BookStoreRuntimeException;

import java.util.Collection;
import java.util.List;

import javax.persistence.EntityManager;

/**
 * Simple BookManager class, creates, modifies, deletes books in the database
 * using JPA.
 * 
 * @author Mark Czotter
 * 
 */
public class BookManager implements IBookManager {

	/* (non-Javadoc)
	 * @see hu.bme.mit.szolgint.osgi.demo.IBookManager#createBook(java.lang.String, int)
	 */
	@Override
	public void createBook(String title, int price, long authorId) {
		EntityManager em = EntityManagerProvider.getEntityManager();
		em.getTransaction().begin();
		//Author author = em.find(Author.class, authorId);
		// create a new book entity and persist it
		Book book = new Book();
		book.setTitle(title);
		book.setPrice(price);
		em.persist(book);
		em.getTransaction().commit();
		em.close();
	}

	/* (non-Javadoc)
	 * @see hu.bme.mit.szolgint.osgi.demo.IBookManager#getBooks()
	 */
	@Override
	public Collection<Book> getBooks() {
		EntityManager em = EntityManagerProvider.getEntityManager();
		List results = em.createNamedQuery(Book.GET_BOOKS).getResultList();
		em.close();
		return results;
	}
	
	/* (non-Javadoc)
	 * @see hu.bme.mit.szolgint.osgi.demo.IBookManager#deleteBook(long)
	 */
	@Override
	public void deleteBook(long bookId) {
		// TODO implement me
	}
	
	/* (non-Javadoc)
	 * @see hu.bme.mit.szolgint.osgi.demo.IBookManager#buyBook(java.lang.String, long)
	 */
	@Override
	public void buyBook(String username, long bookId) throws BookStoreRuntimeException {
		EntityManager em = EntityManagerProvider.getEntityManager();
		em.getTransaction().begin();
		User user = em.find(User.class, username);
		if (user == null) {
			throw new BookStoreRuntimeException("User with " + username + " not registered!");
		}
		Book book = em.find(Book.class, bookId);
		if (book == null) {
			throw new BookStoreRuntimeException("Book with id " + bookId + " not found in the BookStore!");
		}
		user.getOwnedBooks().add(book);
		book.getOwners().add(user);
		em.merge(user);
		em.merge(book);
		em.getTransaction().commit();
		em.close();
	}

	@Override
	public List<Book> searchBook(String searchTitle) {
		EntityManager em = EntityManagerProvider.getEntityManager();
		List results = em.createNamedQuery(Book.SEARCH_BOOK).setParameter("searchTitle", "%"+searchTitle.toUpperCase()+"%").getResultList();
		em.close();
		return results;
	}

}
