package hu.bme.mit.szolgint.osgi.demo;

import hu.bme.mit.szolgint.osgi.demo.entities.EntityManagerProvider;
import hu.bme.mit.szolgint.osgi.demo.entities.User;
import hu.bme.mit.szolgint.osgi.demo.exceptions.BookStoreRuntimeException;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

/**
 * Simple {@link IUserManager} implementation.
 * 
 * @author Mark Czotter
 * 
 */
public class UserManager implements IUserManager {

	/* (non-Javadoc)
	 * @see hu.bme.mit.szolgint.osgi.demo.IUserManager#getUsers()
	 */
	@Override
	public List<User> getUsers() throws BookStoreRuntimeException {
		EntityManager em = EntityManagerProvider.getEntityManager();
		List users = em.createNamedQuery(User.GET_USERS).getResultList();
		em.close();
		return users;
	}

	/* (non-Javadoc)
	 * @see hu.bme.mit.szolgint.osgi.demo.IUserManager#register(java.lang.String)
	 */
	@Override
	public void register(String username) throws BookStoreRuntimeException {
		if (username == null || username.isEmpty()) {
			throw new BookStoreRuntimeException("Invalid username!");
		}
		EntityManager em = EntityManagerProvider.getEntityManager();
		em.getTransaction().begin();
		User existingUser = em.find(User.class, username);
		if (existingUser == null) {
			User user = new User();
			user.setUsername(username);
			em.persist(user);
			em.getTransaction().commit();
			em.close();
		} else {
			em.getTransaction().rollback();
			em.close();
			throw new BookStoreRuntimeException(
					"Username already taken by another User!");
		}
	}
	
	/* (non-Javadoc)
	 * @see hu.bme.mit.szolgint.osgi.demo.IUserManager#getUser(java.lang.String)
	 */
	@Override
	public User getUser(String username) throws BookStoreRuntimeException {
		EntityManager em = EntityManagerProvider.getEntityManager();
		return em.find(User.class, username);
	}

}
