package hu.bme.mit.szolgint.osgi.demo.rest;

import hu.bme.mit.szolgint.osgi.demo.BookManager;
import hu.bme.mit.szolgint.osgi.demo.IBookManager;
import hu.bme.mit.szolgint.osgi.demo.IUserManager;
import hu.bme.mit.szolgint.osgi.demo.UserManager;
import hu.bme.mit.szolgint.osgi.demo.entities.Author;
import hu.bme.mit.szolgint.osgi.demo.entities.Book;
import hu.bme.mit.szolgint.osgi.demo.entities.User;

import java.util.Collection;
import java.util.List;

import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;

@Path("/bookstore")
public class BookStoreRestService implements IBookStoreService {

	private IBookManager bookManager = new BookManager();
	private IUserManager userManager = new UserManager();

	@GET
	public String echo() {
		return "This is a OSGi JAX-RS implementation Demo.";
	}

	@PUT
	@Path("/books/{title}/{price}/{authorId}")
	@Produces("text/plain")
	@Override
	public String addBook(@PathParam("title") String title,
			@PathParam("price") int price, @PathParam("authorId") long authorId) {
		if (bookManager == null) {
			return "Not connected to BookManager!";
		}
		try {
			bookManager.createBook(title, price, authorId);
		} catch (Exception e) {
			e.printStackTrace();
			return e.getMessage();
		}
		return "Successfully added a book to bookStore!";
	}

	@GET
	@Path("/books")
	@Produces("text/plain")
	@Override
	public String listBooks() {
		try {
			if (bookManager == null) {
				return "Not connected to bookstore!";
			}
			Collection<Book> books = this.bookManager.getBooks();
			if (books.isEmpty()) {
				return "No books in the bookstore!";
			}
			return printBooks(books);
		} catch (Exception e) {
			e.printStackTrace();
			return e.getMessage();
		}
	}

	@PUT
	@Path("/users/{username}")
	@Produces("text/plain")
	@Override
	public String register(@PathParam("username") String username) {
		if (userManager == null) {
			return "Not connected to UserManager!";
		}
		try {
			userManager.register(username);
		} catch (Exception e) {
			e.printStackTrace();
			return e.getMessage();
		}
		return "Registration was successful!";
	}

	@GET
	@Path("/users")
	@Produces("text/plain")
	@Override
	public String listUsers() {
		if (userManager == null) {
			return "Not connected to UserManager!";
		}
		try {
			List<User> users = this.userManager.getUsers();
			if (users.isEmpty()) {
				return "No registered Users!";
			}
			StringBuilder result = new StringBuilder();
			result.append("Users\n");
			for (User user : users) {
				result.append(user.getUsername());
				Collection<Book> ownedBooks = user.getOwnedBooks();
				if (!ownedBooks.isEmpty()) {
					result.append("   Owned Books: ");
					for (Book book : ownedBooks) {
						result.append(" " + book.getTitle());
					}
				}
				result.append("\n");
			}
			return result.toString();
		} catch (Exception e) {
			e.printStackTrace();
			return e.getMessage();
		}
	}

	@POST
	@Path("/buy/{username}/{bookId}")
	@Produces("text/plain")
	@Override
	public String buyBook(@PathParam("username") String username, @PathParam("bookId") long bookId) {
		if (bookManager == null) {
			return "Not connected to bookstore!";
		}
		try {
			this.bookManager.buyBook(username, bookId);
			return "Order was successful!";
		} catch (Exception e) {
			e.printStackTrace();
			return e.getMessage();
		}
	}

	@GET
	@Path("/searchBook/{searchTitle}")
	@Produces("text/plain")
	@Override
	public String searchBook(@PathParam("searchTitle") String searchTitle) {
		if (bookManager == null) {
			return "Not connected to bookstore!";
		}
		try {
			Collection<Book> books = this.bookManager.searchBook(searchTitle);
			if (books.isEmpty()) {
				return "No books in the bookstore!";
			}
			return printBooks(books);
		} catch (Exception e) {
			e.printStackTrace();
			return e.getMessage();
		}
	}

	private String printBooks(Collection<Book> books) {
		StringBuilder result = new StringBuilder();
		result.append("Books\n");
		for (Book book : books) {
			result.append(book.getId());
			result.append(", ");
			result.append(book.getTitle());
			result.append(",  ");
			result.append(book.getPrice());
			result.append(",  ");
			for (Author author : book.getAuthors()) {
				result.append(author.getFirstName() + author.getLastName());
				result.append(",");
			}
			result.append("\n");
			Collection<User> owners = book.getOwners();
			result.append("   Owners: ");
			for (User owner : owners) {
				result.append(" " + owner.getUsername());
			}
			result.append("\n");
		}
		return result.toString();
	}

	@GET
	@Path("/books/price/{title}")
	@Produces("text/plain")
	@Override
	public String getPrice(String title) {
		try {
			List<Book> books = this.bookManager.searchBook(title);
			if (!books.isEmpty()) {
				if (books.size() == 1) {
					Book book = books.get(0);
					return ""+book.getPrice();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return "ERROR";
	}
	
//	public synchronized void setBookManager(IBookManager bookManager) {
//		this.bookManager = bookManager;
//	}
//
//	public synchronized void unsetBookManager(IBookManager bookManager) {
//		if (this.bookManager == bookManager) {
//			this.bookManager = null;
//		}
//	}
//	
//	public synchronized void setUserManager(IUserManager userManager) {
//		this.userManager = userManager;
//	}
//
//	public synchronized void unsetUserManager(IUserManager userManager) {
//		if (this.userManager == userManager) {
//			this.userManager = null;
//		}
//	}

}
